% Copyright (c) <2014>, <Boston University>
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
% 1. Redistributions of source code must retain the above copyright
%    notice, this list of conditions and the following disclaimer.
% 2. Redistributions in binary form must reproduce the above copyright
%    notice, this list of conditions and the following disclaimer in the
%    documentation and/or other materials provided with the distribution.
% 3. All advertising materials mentioning features or use of this software
%    must display the following acknowledgement:
%    This product includes software developed by the <organization>.
% 4. Neither the name of the <organization> nor the
%    names of its contributors may be used to endorse or promote products
%    derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY <COPYRIGHT HOLDER> ''AS IS'' AND ANY
% EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%--------------------------------------------------------------------------
function [V S SS] = varianceMap(I, radius)
%--------------------------------------------------------------------------
% History Log
%--------------------------------------------------------------------------
%
% 8/12/2014 - Boston University's Image and Video Computing group released
% this code
%--------------------------------------------------------------------------
%   int x1(iX-radius);
%   int x2(iX+radius+1);
%   int y1(iY-radius);
%   int y2(iY+radius+1);
% 
%   double numPix(radius*2+1);
%   numPix *= numPix;
% 
%   double sum( iSum[y2][x2]-iSum[y1][x2]-iSum[y2][x1]+iSum[y1][x1]);
%   double sumsq(iSq[y2][x2]- iSq[y1][x2]- iSq[y2][x1]+ iSq[y1][x1]);
%   double mean=sum/numPix;
%   double var = sumsq/numPix - mean*mean;

S = cumsum(cumsum(I,1),2);
SS = cumsum(cumsum(I.*I,1),2);
V = zeros(size(I));

width = size(I,2);
height = size(I,1);
x = radius+2:width-radius;
y = radius+2:height-radius;

diameter = 2*radius+1;
Uy = 1:height-diameter;
Ly = diameter+1:height;
Lx = 1:width-diameter;
Rx = diameter+1:width;

S(y,x) = S(Ly,Rx) - S(Uy,Rx) - S(Ly,Lx) + S(Uy,Lx);
SS(y,x) = SS(Ly,Rx) - SS(Uy,Rx) - SS(Ly,Lx) + SS(Uy,Lx);

numPix = diameter^2;

V(y,x) = SS(y,x) / numPix - (S(y,x)/numPix).^2;
V = sqrt(V);
