function [files totalFiles] = checkFoldersAndFiles(root, subdirs, files)
% [files totalFiles] = checkFoldersAndFiles(root, subdirs, files)
% checks for the existence of the subdirectories underneath the root. And
% in each subdirectory, checks for the existence of a set of files
%
% subdirs is a cell array of strings of the names of the subdirectories to
% look for
%
% files is a cell array where each cell contains a cell array of strings of
% the filenames to check for underneath the corresponding subdirectory.

if ~exist(root, 'dir')
    error('Root directory %s does not exist', root);
end
if isempty(files)
    files = cell(size(subdirs,1),1);
end
totalFiles = 0;

for d=1:size(subdirs)
    path = [root subdirs{d}];
    if ~exist(path, 'dir')
        error('Dataset %s does not exist under root %s', subdirs{d}, root{s});
    end
    if ~isempty(files{d})
        for f=1:size(files{d},1)
            file=[path files{d}{f}];
            if ~exist(file, 'file')
                %error('Missing file %s under path %s', files{d}{f}, path);
                warning('Missing file %s under path %s', files{d}{f}, path);
            end
        end
    else
        files{d} = listdir(path);
    end
    totalFiles = totalFiles + size(files{d},1);
end
end
